# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng.protocol import VoidProtocol


class AlereProtocol(VoidProtocol):
    ACK = b"""<?xml version="1.0" encoding="utf-8"?><ACK.R01><HDR><HDR.control_id V="%(control_id)s"/><HDR.version_id V="%(version_id)s"/><HDR.creation_dttm V="%(creation_dttm)s"/></HDR><ACK><ACK.type_cd V="%(type_cd)s"/><ACK.ack_control_id V="%(ack_control_id)s"/></ACK></ACK.R01>"""
    REQ = b"""<?xml version="1.0" encoding="utf-8"?><REQ.R01><HDR><HDR.control_id V="%(control_id)s"/><HDR.version_id V="%(version_id)s"/><HDR.creation_dttm V="%(creation_dttm)s"/></HDR><REQ><REQ.request_cd V="%(request_cd)s"/></REQ></REQ.R01>"""
    END = b"""<?xml version="1.0" encoding="utf-8"?><END.R01><HDR><HDR.control_id V="%(control_id)s"/><HDR.version_id V="%(version_id)s"/><HDR.creation_dttm V="%(creation_dttm)s"/></HDR><TRM><TRM.reason_cd V="NRM"/></TRM></END.R01>"""

    def get_ack_message(self, payload, date=None, control_id=None):
        payload_string = payload.decode("utf-8")
        control_id = self.get_control_id(payload_string)
        if "<OBS.R01>" in payload_string or "<OBS.R02>" in payload_string:
            control_id = str(int(control_id) + 1)
        control_id = control_id.encode("utf_8")
        date = date or datetime.datetime.now()
        context = {
            b"control_id": control_id,
            b"version_id": b"POCT1",
            b"creation_dttm": date.strftime("%Y-%m-%dT%H:%M:%S%z").encode(),
            b"type_cd": b"AA",
            b"ack_control_id": control_id,
        }
        return self.ACK % context

    def get_req_message(self, payload, date=None, control_id=None):
        payload_string = payload.decode("utf-8")
        control_id = str(int(self.get_control_id(payload_string)) + 1).encode("utf_8")
        date = date or datetime.datetime.now()
        context = {
            b"control_id": control_id,
            b"version_id": b"POCT1",
            b"creation_dttm": date.strftime("%Y-%m-%dT%H:%M:%S%z").encode(),
            b"request_cd": b"ROBS",
        }
        return self.REQ % context

    def get_end_message(self, payload, date=None, control_id=None):
        payload_string = payload.decode("utf-8")
        control_id = str(int(self.get_control_id(payload_string)) + 1).encode("utf_8")
        date = date or datetime.datetime.now()
        context = {
            b"control_id": control_id,
            b"version_id": b"POCT1",
            b"creation_dttm": date.strftime("%Y-%m-%dT%H:%M:%S%z").encode(),
        }
        return self.END % context

    def recv_trames(self):
        data = self.read(size=4096)
        data_lines = data.splitlines()
        for line in data_lines:
            if b"<HEL.R01>" in line or b"<OBS.R01>" in line or b"<OBS.R02>" in line:
                self.write(self.get_ack_message(data))
                break
            elif b"<DST.R01>" in line:
                self.write(self.get_ack_message(data))
                self.write(self.get_req_message(data))
                break
            elif b"<EOT.R01>" in line:
                self.write(self.get_end_message(data))
                break
        to_replace = [b"<A>", b"<lf>"]
        for i in to_replace:
            data = data.replace(i, b"\r")
        return data

    def get_control_id(self, splitted_string):
        striped_string = splitted_string.strip()
        hdr_index = striped_string.find('<HDR.control_id V="')
        hdr_string = striped_string[hdr_index : (hdr_index + 25)]
        left_index = hdr_string.find('V="') + 3
        right_index = hdr_string.find('"/>')
        return hdr_string[left_index:right_index]
