# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import time

from driverng.protocol.mllp import MLLPProtocol
from driverng.utils import FS, VT


class CobasPureProtocol(MLLPProtocol):
    HL7_ACK = b"""MSH|^~\\&|%(sender_id)s||%(receiver_id)s||%(date)s||%(message_type)s|%(control_id)s|P|2.5.1|||||UNICODE UTF-8|||LAB-29^IHE\rMSA|AA|%(ack_id)s"""
    HL7_UACK = b"""MSH|^~\\&|%(sender_id)s||%(receiver_id)s||%(date)s||%(message_type)s|%(control_id)s|P|2.5.1|||||UNICODE UTF-8|||ROC-02^ROCHE\rMSA|AA|%(ack_id)s"""
    HL7_QACK = b"""MSH|^~\\&|%(sender_id)s||%(receiver_id)s||%(date)s||%(message_type)s|%(control_id)s|P|2.5.1|||||UNICODE UTF-8|||LAB-27R^ROCHE\rMSA|AA|%(ack_id)s\rQAK|%(query_tag)s|OK|%(qpd_identifier)s\rQPD|%(qpd_identifier)s|%(query_tag)s|%(qpd_sample_id)s|%(qpd_rack)s|%(qpd_position)s|||||%(specimen_type)s|%(container_type)s|%(priority)s"""

    def get_ack_message(self, payload, date=None, control_id=None):
        splitted_payload = payload.splitlines()
        splitted = payload.splitlines()[0].split(b"|")
        query_tag = b""
        qpd_identifier = b""
        qpd_sample_id = b""
        qpd_rack = b""
        qpd_position = b""
        specimen_type = b""
        container_type = b""
        priority = b""
        for line in splitted_payload:
            splitted_qpd = b""
            if b"QPD" in line:
                splitted_qpd = line.split(b"|")
                query_tag = splitted_qpd[2]
                qpd_identifier = splitted_qpd[1]
                qpd_sample_id = splitted_qpd[3]
                qpd_rack = splitted_qpd[4]
                qpd_position = splitted_qpd[5]
                specimen_type = splitted_qpd[10]
                container_type = splitted_qpd[11]
                priority = splitted_qpd[12]
        date = date or datetime.datetime.now()
        type_message = splitted[8]
        control_id = b"%d" % int(time.time() * 1000) if not control_id else control_id
        context = {
            b"sender_id": splitted[4],
            b"receiver_id": splitted[2],
            b"date": date.strftime("%Y%m%d%H%M%S").encode(),
            b"message_type": self.ACK_MESSAGE_TYPE[self.is_automate],
            b"control_id": control_id,
            b"ack_id": splitted[9],
            b"query_tag": query_tag,
            b"qpd_identifier": qpd_identifier,
            b"qpd_sample_id": qpd_sample_id,
            b"qpd_rack": qpd_rack,
            b"qpd_position": qpd_position,
            b"specimen_type": specimen_type,
            b"container_type": container_type,
            b"priority": priority,
        }
        if type_message.startswith(b"QBP"):
            context[b"message_type"] = b"RSP^K11^RSP_K11"
            return CobasPureProtocol.HL7_QACK % context
        elif type_message.startswith(b"ESU"):
            context[b"message_type"] = b"ACK^U01^ACK"
            return CobasPureProtocol.HL7_UACK % context
        elif type_message.startswith(b"INU"):
            context[b"message_type"] = b"ACK^U05^ACK"
            return CobasPureProtocol.HL7_UACK % context
        else:
            context[b"message_type"] = b"ACK^R22^ACK"
            return CobasPureProtocol.HL7_ACK % context

    def recv_trames(self, timeout=None):
        data = self.mllp_readline(timeout)
        messages = []
        acks = []
        for d in data.split(b"\x1c\x0d"):
            if d:
                payload = self.get_payload(d)
                self.logger.info("recv trames: %s ", payload)
                ack = self.get_ack_message(payload)
                if ack:
                    acks.append(ack)
                messages.append(payload)
        for ack in acks:
            self.write(self.build_trames(ack))
        return messages

    def build_trames(self, message):
        # <vt><payload><fs><cr>
        msg = b""
        for m in message.split(b"MSH|"):
            if m:
                msg += b"%c%s\r%c\r" % (VT, b"\r".join((b"MSH|" + m).splitlines()), FS)
        return msg
